#include <Bounce2.h>
#include "MIDIUSB.h"
#define CHANNEL 0
int ButtonPin [4] = {2, 3, 4, 5};
int PotPin [4] = {A0, A1, A2, A3};
char *pot[4] = {"Pot1", "Pot2", "Pot3", "Pot4"};
int ButtonNote [4] = {60, 61, 62, 63};
int PotNote [4] = {17, 18, 19, 20};
Bounce Btn1 = Bounce(ButtonPin[0], 5);
Bounce Btn2 = Bounce(ButtonPin[1], 5);
Bounce Btn3 = Bounce(ButtonPin[2], 5);
Bounce Btn4 = Bounce(ButtonPin[3], 5);
uint8_t Pot1, Pot2, Pot3, Pot4, prevPot1, prevPot2, prevPot3, prevPot4;

void noteOn(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOn = {0x09, 0x90 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOn);
  MidiUSB.flush();
}
void noteOff(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOff = {0x08, 0x80 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOff);
  MidiUSB.flush();
}
void controlChange(byte channel, byte control, byte value) {
  // First parameter is the event type (0x0B = control change).
  // Second parameter is the event type, combined with the channel.
  // Third parameter is the control number number (0-119).
  // Fourth parameter is the control value (0-127).
  midiEventPacket_t event = {0x0B, 0xB0 | channel, control, value};
  MidiUSB.sendMIDI(event);
  MidiUSB.flush();
}

void setup() {
  Serial.begin(115200);
  for (int i = 0; i < 4; i++) {
    pinMode(ButtonPin[i], INPUT_PULLUP);
    pot[i] = map(analogRead(PotPin[i]), 0, 1023, 0, 127);
  }
}

void loop() {
  Btn1.update();
  Btn2.update();
  Btn3.update();
  Btn4.update();

  Pot1 = map(analogRead(PotPin[0]), 0, 1023, 0, 127);
  if (prevPot1 != Pot1) {
    controlChange(CHANNEL, PotNote[0], Pot1);
    prevPot1 = Pot1;
  }
  Pot2 = map(analogRead(PotPin[1]), 0, 1023, 0, 127);
  if (prevPot2 != Pot2) {
    controlChange(CHANNEL, PotNote[1], Pot2);
    prevPot2 = Pot2;
  }
  Pot3 = map(analogRead(PotPin[2]), 0, 1023, 0, 127);
  if (prevPot3 != Pot3) {
    controlChange(CHANNEL, PotNote[2], Pot3);
    prevPot3 = Pot3;
  }
  Pot4 = map(analogRead(PotPin[3]), 0, 1023, 0, 127);
  if (prevPot4 != Pot4) {
    controlChange(CHANNEL, PotNote[3], Pot4);
    prevPot4 = Pot4;
  }
  if (Btn1.fallingEdge())
    noteOn(CHANNEL, ButtonNote[0], 127);
  else if (Btn1.risingEdge())
    noteOff(CHANNEL, ButtonNote[0], 0);
  else if (Btn2.fallingEdge())
    noteOn(CHANNEL, ButtonNote[1], 127);
  else if (Btn2.risingEdge())
    noteOff(CHANNEL, ButtonNote[1], 0);
  else if (Btn3.fallingEdge())
    noteOn(CHANNEL, ButtonNote[2], 127);
  else if (Btn3.risingEdge())
    noteOff(CHANNEL, ButtonNote[2], 0);
  else if (Btn4.fallingEdge())
    noteOn(CHANNEL, ButtonNote[3], 127);
  else if (Btn4.risingEdge())
    noteOff(CHANNEL, ButtonNote[3], 0);
}
